<ul id="navbar">
 <li><a href="template:///home/main.html">Home</a></li>
 <li><a href="template:///flickr/main.html">flickr</a></li>
 <li><a href="template:///identica/main.html">identi.ca</a></li>
 <li><a href="template:///settings/main.html">Settings</a></li>
</ul>
